@@ -1,11 +1,60 @@
-"""A Python implementation of the EMuLSion framework.
-
-(Epidemiologic MUlti-Level SImulatiONs).
+"""A Python implementation of the EMuLSion framework (Epidemiologic
+MUlti-Level SImulatiONs).
 
 Classes and functions for entities management.
-
 """
 
+
+# EMULSION (Epidemiological Multi-Level Simulation framework)
+# ===========================================================
+# 
+# Contributors and contact:
+# -------------------------
+# 
+#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Yu-Lin Huang
+#     - Vianney Sicard
+#     - Sandie Arnoux
+#     - Gaël Beaunée
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+# 
+#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
+# 
+# 
+# How to cite:
+# ------------
+# 
+#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
+#     Sustainability of Complex Epidemiological Models through a Generic
+#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
+#     International Joint Conference on Artificial Intelligence (IJCAI),
+#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+# 
+# 
+# License:
+# --------
+# 
+#    Copyright 2016 INRA and Univ. Lille
+# 
+#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+# 
+#    Agence pour la Protection des Programmes,
+#    54 rue de Paradis, 75010 Paris, France
+# 
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+# 
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+
+
 from   collections               import OrderedDict
 
 from   emulsion.agent.core       import EmulsionAgent
@@ -30,9 +79,11 @@ class AtomAgent(EmulsionAgent):
     def __init__(self, **others):
         super().__init__(**others)
         self.statevars.population = 1
+        self.stochastic = True
         self._host = OrderedDict()
         if 'host' in others:
             self.add_host(others['host'])
+
     def __len__(self):
         return 1
 
@@ -40,7 +91,7 @@ class AtomAgent(EmulsionAgent):
         """Return the population (1) of the current unit.
 
         """
-        return {'population': 1}
+        return ('population', 1)
 
     def add_host(self, host):
         """Add the specified host to the current AtomAgent, associated
@@ -58,7 +109,8 @@ class AtomAgent(EmulsionAgent):
         if keys is None:
             del self._host[host.keys]
         else:
-            del self._host[keys]
+            if keys in self._host:
+                del self._host[keys]
 
     def get_host(self, key='MASTER'):
         """Retrieve the host of the current AtomAgent identified by the
@@ -67,7 +119,7 @@ class AtomAgent(EmulsionAgent):
         """
         return self._host[key]
 
-    def clone(self, prototype=None, **others):
+    def clone(self, prototype=None, custom_prototype=None, **others):
         """Make a copy of the current compartment with the specified
         observable/value settings. If a prototype is provided, it is
         applied to the new atom.
@@ -78,7 +130,9 @@ class AtomAgent(EmulsionAgent):
         new_atom.level = self.level
         new_atom.statevars.update(**others)
         if prototype is not None:
-            new_atom.apply_prototype(prototype)
+            new_atom.apply_prototype(name=prototype)
+        elif custom_prototype is not None:
+            new_atom.apply_prototype(prototype=custom_prototype)
         return new_atom
 
 
