@@ -1,17 +1,65 @@
-"""A Python implementation of the EMuLSion framework.
-
-(Epidemiologic MUlti-Level SImulatiONs).
+"""A Python implementation of the EMuLSion framework (Epidemiologic
+MUlti-Level SImulatiONs).
 
 Classes and functions for entities management.
-
 """
 
 
+# EMULSION (Epidemiological Multi-Level Simulation framework)
+# ===========================================================
+# 
+# Contributors and contact:
+# -------------------------
+# 
+#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Yu-Lin Huang
+#     - Vianney Sicard
+#     - Sandie Arnoux
+#     - Gaël Beaunée
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+# 
+#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
+# 
+# 
+# How to cite:
+# ------------
+# 
+#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
+#     Sustainability of Complex Epidemiological Models through a Generic
+#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
+#     International Joint Conference on Artificial Intelligence (IJCAI),
+#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+# 
+# 
+# License:
+# --------
+# 
+#    Copyright 2016 INRA and Univ. Lille
+# 
+#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+# 
+#    Agence pour la Protection des Programmes,
+#    54 rue de Paradis, 75010 Paris, France
+# 
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+# 
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+
+
 import numpy                     as np
 
 from   emulsion.agent.core       import GroupAgent
 from   emulsion.agent.exceptions import InvalidCompartmentOperation
-# from   emulsion.tools.misc       import aggregate_probabilities
+from   emulsion.tools.misc       import POPULATION
 
 class Compartment(GroupAgent):
     """An Compartment is a compartment which does not
@@ -32,7 +80,7 @@ class Compartment(GroupAgent):
         """Return the population of the current unit.
 
         """
-        return {'population': self.statevars.population}
+        return ('population', self.statevars.population)
 
     def add(self, population):
         """Add the specified population to the current population of
@@ -65,6 +113,10 @@ class Compartment(GroupAgent):
         quantity = min(population, self.statevars.population)
         super().move_to(other_unit, population=quantity, state_machine=state_machine, **others)
 
+    @property
+    def population(self):
+        return self.statevars.population
+
     def clone(self, **others):
         """Make a copy of the current compartment with the specified
         observable/value settings. The new content is empty.
@@ -79,20 +131,22 @@ class Compartment(GroupAgent):
         return new_comp
 
     def next_states(self, states, values, populations, actions, method=None):
-        """Compute the population moving from the current compartment
-        to each of the destination states, handling the values
-        according the the specified method. Values can be handled
-        either as absolute amounts ('amount' method), as proportions
-        ('rate', in a deterministic approach) or as probabilities
-        ('proba', in a stochastic approach). Actions are to be
-        performed when changing state. The actual population affected
-        by the transitions is stored in the first element of the
+        """Compute the population moving from the current compartment to each
+        of the destination states, handling the values according the
+        the specified method. Values can be handled either as absolute
+        amounts ('amount' method), as proportions ('rate', in a
+        deterministic approach) or as probabilities ('proba', in a
+        stochastic approach). Actions are to be performed when
+        changing state. The actual population affected by the
+        transitions is stored in the first element of the
         `populations` parameter, as a dictionary: {'population':
         number, 'actions': actions}. Several edges can lead to the
         same state.
 
+        Return a list of tuples:
+          (state, {'population': qty, 'actions:' list of actions})
         """
-        current_pop = populations[0]['population']
+        current_pop = populations[0][POPULATION]
         if method == 'amount':
             # length of values is expected to be the number of output edges
             # retrieve the amount of population exiting
@@ -113,7 +167,8 @@ class Compartment(GroupAgent):
                 # output edges
                 evolution = [(np.exp(rate*self.model.delta_t) - 1) * current_pop
                              for rate in values]
-        return [(self._host.state_machine.states[state],
-                 {'population': qty, 'actions': act})
-                for state, qty, act in zip(states[:-1], evolution, actions[:-1])
-                if qty > 0]
+        result =  [(self._host.state_machine.states[state],
+                    {'population': qty, 'actions': act})
+                   for state, qty, act in zip(states[:-1], evolution, actions[:-1])
+                   if qty > 0]
+        return result
