@@ -1,19 +1,67 @@
-"""A Python implementation of the EMuLSion framework.
-
-(Epidemiologic MUlti-Level SImulatiONs).
+"""A Python implementation of the EMuLSion framework (Epidemiologic
+MUlti-Level SImulatiONs).
 
 Classes and functions for entities management.
-
 """
 
+
+# EMULSION (Epidemiological Multi-Level Simulation framework)
+# ===========================================================
+# 
+# Contributors and contact:
+# -------------------------
+# 
+#     - Sébastien Picault (sebastien.picault@inra.fr)
+#     - Yu-Lin Huang
+#     - Vianney Sicard
+#     - Sandie Arnoux
+#     - Gaël Beaunée
+#     - Pauline Ezanno (pauline.ezanno@inra.fr)
+# 
+#     BIOEPAR, INRA, Oniris, Atlanpole La Chantrerie,
+#     Nantes CS 44307 CEDEX, France
+# 
+# 
+# How to cite:
+# ------------
+# 
+#     S. Picault, Y.-L. Huang, V. Sicard, P. Ezanno (2017). "Enhancing
+#     Sustainability of Complex Epidemiological Models through a Generic
+#     Multilevel Agent-based Approach", in: C. Sierra (ed.), 26th
+#     International Joint Conference on Artificial Intelligence (IJCAI),
+#     AAAI, p. 374-380. DOI: 10.24963/ijcai.2017/53
+# 
+# 
+# License:
+# --------
+# 
+#    Copyright 2016 INRA and Univ. Lille
+# 
+#    Inter Deposit Digital Number: IDDN.FR.001.280043.000.R.P.2018.000.10000
+# 
+#    Agence pour la Protection des Programmes,
+#    54 rue de Paradis, 75010 Paris, France
+# 
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+# 
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+
+
 from   collections               import OrderedDict
 import numpy                     as np
 
 from   sortedcontainers          import SortedSet
 
 from   emulsion.agent.core       import Aggregation
-from   emulsion.tools.misc       import select_random
-
+from   emulsion.tools.misc       import select_random, AGENTS
 
 #   _____ _                 _   __      ___
 #  / ____(_)               | |  \ \    / (_)
@@ -44,7 +92,7 @@ class SimpleView(Aggregation):
         """Return the units contained in the current unit.
 
         """
-        return {'agents': list(self._content)}
+        return ('agents', list(self._content))
 
     def add(self, population):
         """Add the specified population to the current compartment."""
@@ -60,17 +108,19 @@ class SimpleView(Aggregation):
         self._content -= SortedSet(population)
 
     def next_states(self, states, values, populations, actions, method=None):
-        """Compute stochastically the population moving from the
-        current compartment to each of the destination states,
-        according to the values interpreted as probabilities. Actions
-        are to be performed when changing state. The population
-        affected by the transitions is stored in the first element of
-        the `populations` parameter, as a dictionary: {'agents':
-        list_of_agents}. Several edges can lead to the same state.
+        """Compute stochastically the population moving from the current
+        compartment to each of the destination states, according to
+        the values interpreted as probabilities. Actions are to be
+        performed when changing state. The population affected by the
+        transitions is stored in the first element of the
+        `populations` parameter, as a tuple: ('agents',
+        list_of_agents). Several edges can lead to the same state.
 
+        Return a list of tuples:
+          (state, {'agent': list of agents, 'actions': list of actions})
         """
-        origin = populations[0]['agents']
-        modified = SortedSet()
+        # retrieve the list of agents from the tuple at first position
+        origin = populations[0][AGENTS]
         if method == 'amount':
             # length of values is expected to be the number of output edges
             total_value = sum(values)
@@ -80,18 +130,26 @@ class SimpleView(Aggregation):
                                         tuple(v / total_value for v in values) + (0,),
                                         populations, actions, method=None)
             evolution = [int(v) for v in values]
-            print(states, evolution)
+            # print(states, evolution)
         else:
             # values is expected to be the number of output edges + 1
             evolution = np.random.multinomial(len(origin), values)[:-1]
-        next_ = []
+
+        ### now select agents corresponding to the amounts
+        # init change list
+        changes = []
+        # track agents already selected
+        modified = SortedSet()
         for state, qty, act in zip(states[:-1], evolution, actions[:-1]):
             if qty > 0:
+                # pick up agents randomly (not already chosen) in the
+                # origin group
                 moving = select_random(origin, qty, exclude=modified)
                 modified |= SortedSet(moving)
-                next_.append((self._host.state_machine.states[state],
+                # append this subgroup to the change list
+                changes.append((self._host.state_machine.states[state],
                               {'agents': moving, 'actions': act}))
-        return next_
+        return changes
 
 
 
@@ -131,25 +189,26 @@ class AdaptiveView(SimpleView):
 
         """
         if not callable(condition):
-            return self.get_content() if condition else {}
+            return self.get_content() if condition else ()
         result = list(filter(condition, self._content))
-        return {'agents': result} if result else {}
+        return ('agents', result) if result else ()
 
     def next_states(self, states, values, populations, actions, method=None):
-        """Compute stochastically the population moving from each
-        population (dict {'agents': list_of_agents}) in the
-        `compart_pop` parameter, to each of the possible destination
-        states, according to each of the values interpreted as
+        """Compute stochastically the population moving from each population
+        (tuples ('agents': list_of_agents)) in the `populations`
+        parameter, to each of the possible destination states,
+        according to each of the values interpreted as
         probabilities. Actions are to be performed when changing
         state. Several edges can lead to the same state.
 
+        Return a list of tuples:
+          (state, {'agent': list of agents, 'actions': list of actions})
         """
-        modified = SortedSet()
         if len(populations) == 1:
             if method == 'amount':
                 # length of values is expected to be the number of output edges
                 total_value = sum(values)
-                if total_value > len(populations[0]['agents']):
+                if total_value > len(populations[0][AGENTS]):
                     # restart with proportions instead
                     return self.next_states(states,
                                             tuple(v / total_value
@@ -157,20 +216,27 @@ class AdaptiveView(SimpleView):
                                             populations, actions, method=None)
                 values = values + (total_value,)
             populations *= len(states)-1
-        next_ = []
+        # init change list
+        changes = []
+        # track agents already selected
+        modified = SortedSet()
         for state, value, population, act\
           in zip(states[:-1], values[:-1], populations, actions[:-1]):
             if method == 'amount':
                 evolution = int(value)
             else:
-                evolution = np.random.binomial(len(population['agents']), value)
+                evolution = np.random.binomial(len(population[AGENTS]), value)
             if evolution > 0:
-                moving = select_random(population['agents'],
+                ### now select agents corresponding to the amounts
+                # pick up agents randomly (not already chosen) in the
+                # origin group
+                moving = select_random(population[AGENTS],
                                        evolution, exclude=modified)
                 modified |= SortedSet(moving)
-                next_.append((self._host.state_machine.states[state],
+                # append this subgroup to the change list
+                changes.append((self._host.state_machine.states[state],
                               {'agents': moving, 'actions': act}))
-        return next_
+        return changes
 
 
     def evolve(self, machine=None):
@@ -196,8 +262,7 @@ class AdaptiveView(SimpleView):
         intruders = SortedSet()
         for unit in self:
             for observable in self.statevars.observables:
-                if unit.get_information(observable) !=\
-                   self.statevars[observable]:
+                if unit.get_information(observable) != self.statevars[observable]:
                     intruders.add(unit)
                     break
         if intruders:
@@ -239,6 +304,7 @@ class StructuredView(Aggregation):
     def __init__(self, keep_history=True, **others):
         super().__init__(**others)
         self.keep_history = keep_history
+        self.new_population = []
         self._content = OrderedDict()
         self._notifications = {}
 
@@ -246,14 +312,16 @@ class StructuredView(Aggregation):
         return self._content.values().__iter__()
 
     def __getitem__(self, name):
+        if name not in self._content: # DEBUG: should not happen !
+            print(name)
         return self._content[name]
 
     def get_content(self):
-        """Return the units contained in the current unit.
+        """Return the agents contained in the current view.
 
         """
         print('WARNING, should not enter here !!!')
-        return {'agents': list(self._content.values())}
+        return ('agents', list(self._content.values()))
 
     def add(self, population):
         """Add the specified population to the current
@@ -301,6 +369,7 @@ class StructuredView(Aggregation):
                 else:
                     new_pop[key].append(unit)
             for key, unit in new_pop.items():
+                # print(source_observables, key)
                 new_comp = self.get_or_build(key, source=source_compartment)
 #                print('Moving to:', new_comp, '\n\t', [u._agid for u in unit])
                 source_compartment.move_to(new_comp, agents=unit)
@@ -313,10 +382,15 @@ class StructuredView(Aggregation):
         """
         if key not in self._content:
             key_infos = dict(zip(self.keys, key))
-#            print('Adding new comp:', key, key_infos)
+            # print('Adding new comp:', key, key_infos)
             new_compart = source.clone(host=self, **key_infos)
             new_compart.keys = key
-#            print(new_compart.statevars)
+            # if any key for compart is a state marked "autoremove",
+            # then the whole compart is marked "autoremove"
+            new_compart.autoremove = any(hasattr(value, 'state_machine') and\
+                                  value.state_machine.get_property(value.name, 'autoremove')
+                                  for value in key)
+            # print(new_compart, new_compart.autoremove)
             self.add({key: new_compart})
         return self[key]
 
@@ -334,8 +408,8 @@ class StructuredViewWithCounts(StructuredView):
     """
     def __init__(self, allowed_values=None, **others):
         super().__init__(**others)
-        if allowed_values:
-            self.allowed_values = allowed_values
+        # if allowed_values:
+        self.allowed_values = allowed_values
         self.init_counts()
 
     def init_counts(self, index=0):
@@ -395,6 +469,7 @@ class AutoStructuredView(StructuredView):
     def __init__(self, key_variable=None, **others):
         super().__init__(**others)
         self.key_variable = key_variable
+        self.current_key = 0    # auto-increment ID if none specified
 
 
     def add(self, population):
@@ -403,6 +478,12 @@ class AutoStructuredView(StructuredView):
 
         """
         for agent in population:
+            if self.key_variable not in agent.statevars:
+                agent.statevars[self.key_variable] = self.current_key
+                self.current_key += 1
+            else:
+                self.current_key = max(1 + agent.statevars[self.key_variable],
+                                       self.current_key)
             self._content[agent.statevars[self.key_variable]] = agent
 
     def remove(self, population):
